//---------------------------------------------------------
//  RG Basic Tools Collection
//  Copyright © 2003,2004 Ronald D. Grindle
//
//This file is part of the Rslt services.
//
//Rslt services is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//Rslt services is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with Rslt services.  If not, see <http://www.gnu.org/licenses/>.
//---------------------------------------------------------

package com.qanavigation.Basics;

import java.util.ArrayList;
import java.util.Collection;

/**
* Simple, 2-dim array. Jagged arrays not supported.
* Rslt2DArray isA Rslt.
*/
//public class Rslt2DArray extends ArrayList implements IRslt
public class Rslt2DArray extends RsltLst<String[]>
{

	static final long serialVersionUID = 61940005;
	
	protected int m_ColSize;

	//protected Rslt m_Result;
	protected int m_RowSize;

	//-------------------------------------------
	public Rslt2DArray( )
 	{

		//super(); 2004-06-28
		init();
	}
	
	//-------------------------------------------
	public Rslt2DArray( boolean p_isOK )
	{

		init();
		
		if( p_isOK ) {
			
			setOK();
		}
	}

	//-------------------------------------------
	public Rslt2DArray( Collection p_Coll, int p_Cols )
	{

		super( p_Coll );
		
		int rowSize;

		init();

		rowSize = super.size();

		setSize(p_Cols, rowSize);

		setOK();
	}

	// ------------------------------------------
	public Rslt2DArray( int p_Cols )
	{

		//super( ); 2004-06-28
		init();
		setSize( p_Cols, 0 );
	}

	// ------------------------------------------
	public Rslt2DArray( int p_Cols, int p_Rows )
	{

		super( p_Rows );
		init();
		setSize( p_Cols, p_Rows );
	}
	
	/**
	 * 
	 * @param p_Row
	 */
	public boolean add( String[] p_Row ) {
		
		boolean ret;

		ret = super.add( p_Row );
		
		setOK();
		
		return ret;
	}

	/**
	 * Overridden member from ArrayList.
	 * @param p_Col
	 */
	public void addRange( Collection p_Col ) {

		super.addAll( p_Col );
		
		setOK();
	}	

	/**
	 *  @param p_Res
	 * /
	public void addRsltAddInfo( IRslt p_Res ) {

		m_Result.addRsltAddInfo( p_Res );
	}

	/ **
	 * @param p_Info
	 * /
	public void addRsltAddInfo( String p_Info ) {

		m_Result.addRsltAddInfo( p_Info );
	}
	
	/ **
	 * 
	 * @param p_Res
	 * @return
	 * /
	public boolean checkRslt( IRslt p_Res ) {

		if( p_Res.notOK() ) {

			setRsltStat( p_Res );
		}

		return isOK();
	}

	/ **
	 * 
	 * @param p_Res
	 * @param p_EmptyIsOK
	 * @return
	 * /
	public boolean checkRslt( IRslt p_Res, boolean p_EmptyIsOK ) {

		if( p_Res.notOK() ) {

			setRsltStat( p_Res );
		}

		return isOK( p_EmptyIsOK );
	}
	*/
	
	// ---------------------------
	protected boolean getOrder( int p_SrtCol, int[] p_IdxLst, boolean p_Desc, boolean p_Numeric ) {
		
		boolean notSorted;
		boolean swap;
		String entry1;
		String entry2;
		int c, i, x;
		int ia, ib;
		int e1, e2;
		
		e1 = 0;
		e2 = 0;
		
		c = p_SrtCol;
		
		x = p_IdxLst.length;
		
		notSorted = true;
		
		try {

			while( notSorted ) {
	
				notSorted = false;
	
				if( x > 1 ) {
					
					ia = p_IdxLst[0];
	
					entry1 = getCell( c, ia );
					
					if( p_Numeric ) {
						
						if( entry1.length() == 0 ) {
							
							e1 = Integer.MIN_VALUE;
						}
						else {
					
							e1 = Integer.valueOf( entry1 );
						}
					}
			
					for( i = 1; i < x; i++ ) {
			
						ib = p_IdxLst[i];
						entry2 = getCell( c, ib );
						
						if( p_Numeric ) {
							
							if( entry2.length() == 0 ) {
								
								e2 = Integer.MIN_VALUE;
							}
							else {
						
								e2 = Integer.valueOf( entry2 );
							}
							
							if( p_Desc ) {
								
								swap = e1 < e2;						
							}
							else {
								
								swap = e1 > e2;
							}						
						}
						else {
						
							if( p_Desc ) {
								
								swap = ( entry1.compareTo( entry2 ) < 0 );						
							}
							else {
								
								swap = ( entry1.compareTo( entry2 ) > 0 );
							}
						}
	
						if( swap ) {
			
							p_IdxLst[i-1] = ib;
							p_IdxLst[i] = ia;
			
							notSorted = true;
						}
						else {
							
							entry1 = entry2;
							ia = ib;
							e1 = e2;
						}					
					}
				}
			}
		}
		catch( Exception exc ) {
			
			notSorted = true;
		}
	
		return notSorted;
	}	

	/**
	 * 
	 * @param p_col
	 * @param p_row
	 * @return
	 */
	public String getCell( int p_col, int p_row ) {

		String[] rowVal;
	
		rowVal = (String[])get( p_row );

		return rowVal[p_col];
	}

	/** 
	* The allocated size.
	*/
	public int getColSize() {

		int cols;

		cols = m_ColSize;

		return cols;
	}
	
	/**
	 * 
	 * @param p_col
	 * @param p_row
	 * @return
	 */
	public String[] getRow( int p_row ) {

		String[] row;
	
		row = (String[])get( p_row );

		return row;
	}

	/**
	 * 
	 * @return the actual number of rows
	 */
	public int getRowNum() {

		int rows;

		rows = size();

		return rows;
	}

	/**
	 * 
	 * /
	public IRslt getRslt() {

		return m_Result.getRslt();
	}

	/ **
	 * 
	 * /
	public ArrayList<String> getRsltAddInfo() {

		return m_Result.getRsltAddInfo();
	}

	/ **
	* Last (error) message.
	* /
	public String getRsltLastMsg() {

		return m_Result.getRsltLastMsg();
	}

	/ **
	 * 
	 * /
	public int getRsltStatVal() {
		
		return m_Result.getRsltStatVal();
	}*/

	/** replaced by getRow()
	 * Can not override get( int)
	 * @param p_index
	 * @return
	 * /
	/ *
	public String[]	getStringArr( int p_index ){

		String[] val;
		Object obj;

		obj = get( p_index );

		val = (String[])obj;

		return val;
	}
	*/

	//-------------------------------------------
	protected void init() {

		super.init();
		
		m_Result = new Rslt( this.getClass() );
		
		m_ColSize = 0;
		m_RowSize = 0;
	}
	
	/**
	 * Only merge two tables of the same type!
	 */
	public void merge( Rslt2DArray p_Tbl, int p_Col ) {
		
		boolean notIn;
		int i, k, x, y;
		
		String val, tVal;
		
		x = p_Tbl.getRowNum();
		
		for( i = 0; i < x; ++i ) {
			
			notIn = true;
			
			y = getRowNum();
			
			val = p_Tbl.getCell( p_Col, i );
			
			for( k = 0; k < y; ++k ) {
				
				tVal = getCell( p_Col, k );
				
				if( tVal.compareTo( val ) == 0  ) {
					
					y = k;
					notIn = false;
				}
			}
		 	
			if( notIn ) {
				
				add( p_Tbl.getRow( i ) );
			}
		}
	}
	
	/**
	* Not in error, not empty, not not init.<br>
	* OK and has at least 1 row.
	*/
	public boolean isNotEmpty() {

		boolean isNotEmpty = false;

		int resStat;

		resStat = m_Result.getRsltStatVal();

		if( RsltStatVal.isOK == resStat  ) {

			if( getRowNum() > 0 ) {

				isNotEmpty = true;
			}
		}

		return isNotEmpty;
	}

	/**
	 * 
	 * /
	public boolean isInStat( int p_Stat ) {

		boolean isStat = false;

		int resStat;

		resStat = m_Result.getRsltStatVal();

		if( resStat == p_Stat ) {

			isStat = true;
		}

		return isStat;
	}
	
	/ **
	 * @return
	 * /
	public boolean isOK( ) {
		
		int resStat;

		boolean isOK = false;

		resStat = m_Result.getRsltStatVal();

		if( resStat == RsltStatVal.isOK ) {

			isOK = true;
		}

		return isOK;
	}
	
	/ **
	 * 
	 * @param p_EmptyIsOK
	 * @return
	 * /
	public boolean isOK( boolean p_EmptyIsOK ) {
		
		int resStat;

		boolean isOK;

		resStat = m_Result.getRsltStatVal();
	
		if( (resStat == RsltStatVal.isEmpty) && p_EmptyIsOK ) {
				
				isOK = true;
		}
		else {
			
			isOK = isOK();
		}

		return isOK;
	}
	
	/ **
	 * 
	 * 	@return
	 * /
	public boolean notOK() {

		boolean notOK = !(isOK());

		return notOK;
	}
	
	/ **
	 * 
	 * @param p_EmptyIsOK
	 * @return
	 * /
	public boolean notOK( boolean p_EmptyIsOK ) {

		boolean notOK = !(isOK( p_EmptyIsOK ));

		return notOK;
	}

	/**
	 * 
	 * @return
	 */
	public boolean notOKorEmpty() {

		boolean notOK = true;

		int resStat;
		resStat = m_Result.getRsltStatVal();

		if( RsltStatVal.isOK == resStat ) {

			notOK = false;
		}

		return notOK;
	}
	
	//	-----------------------------------------
	/*
	public String getSelfReflect() {

		String sRefl;
	
		sRefl = m_Result.getSelfReflect( this.getClass() );

		return sRefl;	
	}
	*/

	/**
	 * 
	 */
	public void resetRslt( ) {

		m_Result.resetRslt();
	}

	/**
	 * 
	 * @param p_index
	 * @param p_Value
	 *
	public void set( int p_index, String[] p_Value ){

		super.set(p_index, p_Value);
	} */

	/**
	 * 
	 * /
	public void setInError(String p_ErrMsg) {

		m_Result.setInError( p_ErrMsg );
	}

	/ **
	 * 
	 * /
	public void setOK() {
		
		m_Result.setOK();
	}

	/ **
	 * 
	 * /
	public void setRsltByExcp( Exception p_Excp ) {

		m_Result.setRsltByExcp( p_Excp );
	}

	/ **
	 * 
	 * /
	public void setRsltStat( int p_aResStat ) {
		
		m_Result.setRsltStat( p_aResStat );
	}
	
	/ **
	 * 
	 * /
	public void setRsltStat( int p_aResStat, String p_LastMsg) {
		
		m_Result.setRsltStat( p_aResStat, p_LastMsg );
	}

	/ **
	 * 
	 * /
	public void setRsltStat( IRslt p_aRes ) {

		m_Result.setRsltStat( p_aRes );
	}

	/ **
	 * 
	 * /
	public void setRsltStat( IRslt p_aRes, String p_Err ) {

		m_Result.setRsltStat( p_aRes, p_Err );
	}
	*/
	
	// -----------------------------------------
	protected void setSize( int p_Cols, int p_Rows ) {

		m_ColSize = p_Cols;
		m_RowSize = p_Rows;

		setRsltStat( RsltStatVal.isInit );
				
		//rowCount = 0;
	}
	
	/**
	 * 
	 * @param p_SortCol
	 * @param p_Desc Sorting order.
	 * @param p_Numeric Only integer supported.
	 * @return not sorted
	 */
	public boolean sort( int p_SortCol, boolean p_Desc, boolean p_Numeric ) {
		
		boolean notSorted;
		
		int i, x;
		
		String[] row;
		
		x = getRowNum();
		
		ArrayList<String[]> cpy = new ArrayList<String[]>(this);
		
		int[] idx = new int[x];
		
		for( i = 0; i < x; ++i ) {
			
			idx[i] = i;
		}
		
		notSorted = getOrder( p_SortCol, idx, p_Desc, p_Numeric );
		
		if( !( notSorted ) ) {
			
			this.removeAll( this );

			for( i = 0; i < x; ++i ) {
				
				row = cpy.get( idx[i] );
				add( row );
			}
		}
		
		cpy = null;

		return notSorted;
	}
}

