// -----------------------------
//Copyright © 2008 Ronald D. Grindle
//
//This file is part of the Rslt services.
//
//Rslt services is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//Rslt services is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with Rslt services.  If not, see <http://www.gnu.org/licenses/>.
// -----------------------------

package com.qanavigation.Basics;

import java.util.*;

// ###############################
public class RsltLst<E> extends ArrayList<E> implements IRslt {
	
	static final long serialVersionUID = 61940002;
	
	protected Rslt m_Result;
	
	/**
	 * 
	 */
	public RsltLst() {
		
		super();

		init();
	}

	/**
	 * 
	 * @param p_Coll
	 */
	public RsltLst( Collection<E> p_Coll ) {
	
		super( p_Coll );
		
		init();
	
		setOK();
	}

	/**
	 * 
	 * @param p_Size
	 */
	public RsltLst( int p_Size ) {
	
		super( p_Size );
		
		init();
	}
	
	/**
	 * 
	 * @param p_Col
	 */
	public void addRange( Collection<E> p_Col ) {

		super.addAll( p_Col );
		
		setOK();
	}
	
	/**
	 * Is plugged together to 'p_Class + "." + p_Fnct + "()" + p_AddInf1 + " failed. " + p_AddInf2'.
	 * @param p_Class
	 * @param p_Fnct
	 * @param p_AddInf1
	 * @param p_AddInf2
	 */
	public void addRsltAddFailInfo( String p_Class, String p_Fnct, String p_AddInf1, String p_AddInf2 ) {
		
		//inf = p_Obj.getClass().getDeclaringClass().getName() + " "; Null Pointer
		//p_Obj.getClass().getName() get the top class
		//inf = p_Obj.getClass().getCanonicalName() + " "; not any better
		
		addRsltAddInfo( p_Class + "." + p_Fnct + "()" + p_AddInf1 + " failed. " + p_AddInf2 );
	}
	
	/* Class Information not actual Class but constructed class 
	public void addRsltAddFailInfo( Object p_Class, String p_Fnct, String p_AddInf1, String p_AddInf2 ) {
		
		//inf = p_Obj.getClass().getDeclaringClass().getName() + " "; Null Pointer
		//p_Obj.getClass().getName() get the top class
		//inf = p_Obj.getClass().getCanonicalName() + " "; not any better
		
		Class[] aCls;
		
		//aCls = p_Class.getClass().getClasses();
		aCls = p_Class.getClass().getDeclaredClasses();
		
		int i, x;
		
		x = aCls.length;
		
		for( i = 0; i < x; ++i ) {
		
			addRsltAddInfo( "Class: " + aCls[i].getName() );
			
		}
		
		addRsltAddInfo( "Classes: " + x );
		
		//addRsltAddInfo( "ec " + p_Class.getClass().getEnclosingClass().getName() );
		
		addRsltAddInfo( p_Fnct + "()" + p_AddInf1 + " failed. " + p_AddInf2 );
	}*/


	/**
	 * Copies the additional information from the given IRslt object.
	 * @param p_Res
	 */
	public void addRsltAddInfo( IRslt p_Res ) {
	
		m_Result.addRsltAddInfo( p_Res );
	}
	
	/**
	 * @param p_Info
	 */
	public void addRsltAddInfo( String p_Info ) {
	
		m_Result.addRsltAddInfo( p_Info );
	}	
	
	/**
	 * Adds a value to the array. Sets the object to OK.
	 * @param p_val
	 */
	public void addVal( E p_val ) {
	
		add( p_val );
		
		setOK();
	}
	
	/**
	 * Checks the Status of the given Rslt and its own. Sets itself to the error code of the given Rslt, if notOK.
	 * @param p_Res 
	 * @return isOK(), which also may be the state before the call!  
	 */
	public boolean checkRslt( IRslt p_Res ) {
	
		if( p_Res.notOK() ) {
	
			setRsltStat( p_Res );
		}
	
		return isOK();
	}
	
	/**
	 * Same as checkRslt( IRslt p_Res ) but tolerates if the result or itself isEmpty.
	 * @param p_Res
	 * @param p_EmptyIsOK 
	 * @return isOK(), which also may be the state before the call!  
	 */
	public boolean checkRslt( IRslt p_Res, boolean p_EmptyIsOK ) {
	
		if( p_Res.notOK( p_EmptyIsOK ) ) {
	
			setRsltStat( p_Res );
		}
	
		return isOK( p_EmptyIsOK );
	}


	/**
	 * @return The Rslt object.
	 */
	public IRslt getRslt() {
	
		return m_Result.getRslt();
	}
	

	/**
	 * @return The array of additional information.
	 */
	public ArrayList<String> getRsltAddInfo() {
	
		return m_Result.getRsltAddInfo();
	}
	
	/**
	* Last (error) message.
	*/
	public String getRsltLastMsg() {
	
		return m_Result.getRsltLastMsg();
	}
	
	// --------------------------------------------
	public int getRsltStatVal() {
		
		return m_Result.getRsltStatVal();
	}
	
	/**
	 * @return
	 */
	public String getRsltStatValLbl() {
		
		return m_Result.getRsltStatValLbl();
	}
	
	//---------------------------------------------------------------------------
	protected void init() {
	
		m_Result = new Rslt();
		
		m_Result.setRsltStat( RsltStatVal.isEmpty );
	}
	

	/**
	 * Checks if the Rslt is in expected status.
	 * @param p_Stat
	 * @return
	 */
	public boolean isInStat( int p_Stat ) {
	
		boolean isStat = false;
	
		int resStat;
	
		resStat = m_Result.getRsltStatVal();
	
		if( resStat == p_Stat ) {
	
			isStat = true;
		}
	
		return isStat;
	}
		
	
	/**
	 * 
	 * @return
	 */
	public boolean isOK( ) {
		
		int resStat;

		boolean isOK = false;

		resStat = m_Result.getRsltStatVal();

		if( resStat == RsltStatVal.isOK ) {

			isOK = true;
		}

		return isOK;
	}
	
	/**
	 * Empty is also OK. 
	 * <p>Overides the IRslt method
	* /
	public boolean isOK() {
		
		int resStat;
	
		boolean isOK = false;
	
		resStat = m_Result.getRsltStatVal();
	
		if( (resStat == RsltStatVal.isEmpty) || (resStat == RsltStatVal.isOK) ) {
	
			isOK = true;
		}
	
		return isOK;
	} */
	
	/**
	 * 
	 * @param p_EmptyIsOK
	 * @return
	 */
	public boolean isOK( boolean p_EmptyIsOK ) {
		
		int resStat;
	
		boolean isOK;
	
		resStat = m_Result.getRsltStatVal();
	
		if( (resStat == RsltStatVal.isEmpty) && p_EmptyIsOK ) {
				
				isOK = true;
		}
		else {
			
			isOK = isOK();
		}
	
		return isOK;
	}
	
	/**
	 * Empty passes as OK. 
	* Overides the IRslt method
	*/
	public boolean notOK() {
	
		boolean notOK = !(isOK());
	
		return notOK;
	}
	
	/**
	 * 
	 * @param p_EmptyIsOK
	 * @return
	 */
	public boolean notOK( boolean p_EmptyIsOK ) {
	
		boolean notOK = !(isOK( p_EmptyIsOK ));
	
		return notOK;
	}
	
	/**
	 * "Removes from this list all of the elements whose index is between fromIndex, inclusive, and toIndex, exclusive. Shifts any succeeding elements to the left (reduces their index)."
	 * @param p_StartIdx
	 * @param p_EndIdx
	 */
	public void removeRange( int p_StartIdx, int p_EndIdx ) {
		
		removeRange( p_StartIdx, p_EndIdx );
	}
	
	/**
	 * 
	 */
	public void resetRslt( ) {
	
		m_Result.resetRslt();
	}
	

	/**
	 * Set Rslt to inError status, with message.
	 * @param p_ErrMsg
	 */
	public void setInError(String p_ErrMsg) {
	
		m_Result.setInError( p_ErrMsg );
	}
	

	/**
	 * 
	 */
	public void setOK() {
		
		m_Result.setOK();
	}
	

	/**
	 * @param p_Excp
	 */
	public void setRsltByExcp( Exception p_Excp ) {
	
		m_Result.setRsltByExcp( p_Excp );
	}
	
	/**
	 * p_aResStat
	 */
	public void setRsltStat( int p_aResStat ) {
		
		m_Result.setRsltStat( p_aResStat );
	}
	

	/**
	 * @param p_aResStat
	 * @param p_LastMsg
	 */
	public void setRsltStat( int p_aResStat, String p_ErrMsg ) {
		
		m_Result.setRsltStat( p_aResStat, p_ErrMsg );
	}
	

	/**
	 * @param p_aRes
	 */
	public void setRsltStat( IRslt p_aRes ) {
	
		m_Result.setRsltStat( p_aRes );
	}
	
	/**
	 * @param p_aRes
	 * @param p_Err
	 */
	public void setRsltStat( IRslt p_aRes, String p_ErrMsg ) {
	
		m_Result.setRsltStat( p_aRes, p_ErrMsg );
	}
	
	// end IRslt members ------------------------
}
